/**
 * ==============================================
 *  DQL : Descartes Query Language
 * ==============================================
 *
 * (c) Copyright 2014-2016, by Juergen Walter, Fabian Gorsler, Fabian Brosig and other contributors.
 *
 * Project Info:   http://descartes.tools/dql
 *
 * All rights reserved. This software is made available under the terms of the
 * Eclipse Public License (EPL) v1.0 as published by the Eclipse Foundation
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * This software is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License (EPL)
 * for more details.
 *
 * You should have received a copy of the Eclipse Public License (EPL)
 * along with this software; if not visit http://www.eclipse.org or write to
 * Eclipse Foundation, Inc., 308 SW First Avenue, Suite 110, Portland, 97204 USA
 * Email: license (at) eclipse.org
 *
 * [Java is a trademark or registered trademark of Sun Microsystems, Inc.
 * in the United States and other countries.]
 */
package tools.descartes.dql.core.engine.aggregates.calculators;

import java.math.BigDecimal;

/** 
 * Implementation of an AggregateController to compute the standard deviation
 * 
 * @author Fabian Gorsler <fabian.gorsler@student.kit.edu>
 */
public class StdDevAggregateCalculator extends AbstractSummaryStatisticsAggregateCalculator {

	@Override
	public BigDecimal getResult() {
		if (!isValid()) {
			return BigDecimal.ZERO;
		}

		return new BigDecimal(getStatistics().getStandardDeviation());
	}

	@Override
	public boolean isValid() {
		return !Double.isNaN(getStatistics().getStandardDeviation());
	}

}
