/**
 * ==============================================
 *  DQL : Descartes Query Language
 * ==============================================
 *
 * (c) Copyright 2014-2016, by Juergen Walter, Fabian Gorsler, Fabian Brosig and other contributors.
 *
 * Project Info:   http://descartes.tools/dql
 *
 * All rights reserved. This software is made available under the terms of the
 * Eclipse Public License (EPL) v1.0 as published by the Eclipse Foundation
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * This software is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License (EPL)
 * for more details.
 *
 * You should have received a copy of the Eclipse Public License (EPL)
 * along with this software; if not visit http://www.eclipse.org or write to
 * Eclipse Foundation, Inc., 308 SW First Avenue, Suite 110, Portland, 97204 USA
 * Email: license (at) eclipse.org
 *
 * [Java is a trademark or registered trademark of Sun Microsystems, Inc.
 * in the United States and other countries.]
 */
package tools.descartes.dql.core.engine.observations;

import static tools.descartes.dql.core.engine.observations.TimeModifierConverter.convert;
import tools.descartes.dql.lang.descartesQL.RelativeTimeDurationClause;
import tools.descartes.dql.lang.descartesQL.SampleClause;
import tools.descartes.dql.lang.descartesQL.TimeModifierType;

public class SampleInterpreter {
	private long sampleSize = 0;

	public SampleInterpreter(final SampleClause sampleClause) {
		if (sampleClause != null) {
			interpretSample(sampleClause);
		}
	}

	public long getSampleSize() {
		return sampleSize;
	}

	private void interpretSample(final SampleClause sampleClause) {
		for (final RelativeTimeDurationClause r : sampleClause.getDurationClause()) {
			final int duration = r.getDuration();
			final TimeModifierType timeModifier = r.getTimeModifier();

			sampleSize += duration * convert(timeModifier);
		}
	}
}
