/**
 * ==============================================
 *  DQL : Descartes Query Language
 * ==============================================
 *
 * (c) Copyright 2014-2016, by Juergen Walter, Fabian Gorsler, Fabian Brosig and other contributors.
 *
 * Project Info:   http://descartes.tools/dql
 *
 * All rights reserved. This software is made available under the terms of the
 * Eclipse Public License (EPL) v1.0 as published by the Eclipse Foundation
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * This software is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License (EPL)
 * for more details.
 *
 * You should have received a copy of the Eclipse Public License (EPL)
 * along with this software; if not visit http://www.eclipse.org or write to
 * Eclipse Foundation, Inc., 308 SW First Avenue, Suite 110, Portland, 97204 USA
 * Email: license (at) eclipse.org
 *
 * [Java is a trademark or registered trademark of Sun Microsystems, Inc.
 * in the United States and other countries.]
 */

package tools.descartes.dql.core.engine.util;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Properties;

import tools.descartes.dql.models.mapping.domain.DoF;
import tools.descartes.dql.models.mapping.mapping.EntityMapping;

public class DoFCrossProductHelper {

	Map<DoF, String[]> possibleDofValues;
	List<HashMap<DoF, String>> result;
	int distinctDoFs;
	Map<String, List<Integer>> map;
	EntityMapping request;
	String[] dofss;
	int maxOccurences;
	int mostFrequentDoFIndex;

	/**
	 * Calculates the permutations for DoFs of the EntityMapping
	 * 
	 * @param input
	 * @return List of DoF-combination. Each list element contains one possible
	 *         combination
	 */
	public static List<HashMap<DoF, String>> calculateCrossProduct(EntityMapping input) {
		return (new DoFCrossProductHelper()).calculatePermutations(input);

	}

	private List<HashMap<DoF, String>> calculatePermutations(EntityMapping request) {
		this.request = request;
		map = new HashMap<String, List<Integer>>();
		for (int i = 0; i < request.getDoF().size(); i++) {
			DoF dof = request.getDoF().get(i);
			String key = dof.getIdentifier();
			if (!map.containsKey(key)) {
				List<Integer> list = new LinkedList<Integer>();
				list.add(i);
				map.put(dof.getIdentifier(), list);
			} else {
				List<Integer> list = map.get(key);
				list.add(i);
				map.put(key, list);
			}
		}
		int repeatedDoFs = 0;
		maxOccurences = 1;
		for (String key : map.keySet()) {
			System.out.println("" + key + " " + map.get(key));
			if (map.get(key).size() > 1) {
				maxOccurences = Math.max(map.get(key).size(), maxOccurences);
				mostFrequentDoFIndex = 1;
				repeatedDoFs++;

			}
		}
		distinctDoFs = map.keySet().size();

		if (repeatedDoFs == 0) {
			/** calculate cross product */
			possibleDofValues = getPossibleDofValues(request);
			this.result = new ArrayList<HashMap<DoF, String>>();
			this.recursiveCrossProduct(0, new HashMap<DoF, String>());

		} else {
			/**
			 * Calculate permutations matching recurring DoFs instead of full permutation
			 * Illustrative example:
			 * 
			 * 'cpu cores'<8>,
			 * 
			 * 'users'<50,75,100, 150>, 'Probability dropRequests'<0.0>,
			 * 
			 * 'users'<300>, 'Probability dropRequests'<0.1>,
			 * 
			 * 'users'<400>, 'Probability dropRequests'<0.3>,
			 * 
			 * Generates 6 permutations: <8, (((50|75|100|150),0.0) | (300,0.1) | (400,0.2)>
			 * 
			 */
			this.possibleDofValues = getPossibleDofValues(request);
			this.dofss = (String[]) map.keySet().toArray(new String[map.keySet().size()]);

			this.result = new ArrayList<HashMap<DoF, String>>();
			for (int i = 0; i < maxOccurences; i++) {
				generatePermutations(new HashMap<DoF, String>(), 0, i);
			}
		}
		return result;
	}

	/**
	 * Extracts map containing all Dofs and their possible values
	 */
	private Map<DoF, String[]> getPossibleDofValues(EntityMapping input) {
		Map<DoF, String[]> possibleDofValues = new HashMap<DoF, String[]>();
		ArrayList<DoF> inputDoFs = new ArrayList<DoF>(input.getDoF());
		for (int i = 0; i < inputDoFs.size(); i++) {
			DoF dof = inputDoFs.get(i);
			String[] values = getDoFValues(dof.getProperties());
			possibleDofValues.put(dof, values);
		}

		return possibleDofValues;
	}

	/**
	 * recursive traversal of possible options
	 */
	@SuppressWarnings("unchecked")
	private void recursiveCrossProduct(int counter, HashMap<DoF, String> temporaryResult) {
		if (counter == possibleDofValues.size())
			result.add((HashMap<DoF, String>) temporaryResult.clone());
		else {
			Entry<DoF, String[]> entry = getDoFPair(possibleDofValues, counter);
			for (String string : entry.getValue()) {
				temporaryResult.put(entry.getKey(), string);
				recursiveCrossProduct(counter + 1, temporaryResult);
			}
		}
	}

	private void generatePermutations(HashMap<DoF, String> temporaryResult, int dofNumber, int occurence) {

		if (dofNumber >= distinctDoFs) {
			result.add((HashMap<DoF, String>) temporaryResult.clone());
		} else {
			String key = dofss[dofNumber];
			DoF dof = request.getDoF().get(map.get(key).get(Math.min(occurence, map.get(key).size() - 1)));
			String[] values = possibleDofValues.get(dof);
			dofNumber = dofNumber + 1;
			for (String dofValue : values) {
				temporaryResult.put(dof, dofValue);
				generatePermutations((HashMap<DoF, String>) temporaryResult.clone(), dofNumber, occurence);
			}
		}

	}

	/**
	 * returns element number i for a map, treating it like a map
	 */
	private Entry<DoF, String[]> getDoFPair(Map<DoF, String[]> map, int counter) {
		int i = 0;

		for (Entry<DoF, String[]> entry : map.entrySet()) {
			if (i == counter)
				return entry;
			i++;
		}

		throw new IllegalStateException("Internal Error: not enough objects in map");
	}

	/**
	 * Creates an String[] with the values of the requested DoF. Calculation depends
	 * on if it is an IntervalVariationClause or a ValueVariationClause
	 * 
	 * @param doFProperties
	 * @return DoF values as String[]
	 */
	private static String[] getDoFValues(Properties doFProperties) {
		String arrayString = "";

		if (doFProperties.get("function").equals("iterate")) {
			int current = (Integer) doFProperties.get("start");
			int stop = (Integer) doFProperties.get("end");
			int step = (Integer) doFProperties.get("step");
			List<Integer> values = new LinkedList<Integer>();
			while (current <= stop) {
				values.add(current);
				current = current + step;
			}
			if (values.get(values.size() - 1) < stop && !values.contains(stop)) {
				values.add(stop);
			}
			for (Integer value : values) {
				arrayString = arrayString + String.valueOf(value) + ",";
			}
			arrayString = arrayString.substring(0, arrayString.length() - 1);

		} else if (doFProperties.get("function").equals("fixed")) {
			arrayString = (String) doFProperties.get("value");
			arrayString = arrayString.replace("]", "");
			arrayString = arrayString.replace("[", "");
			arrayString = arrayString.replace(" ", "");
		}
		return arrayString.split(",");
	}

}
