/**
 * ==============================================
 *  DQL : Descartes Query Language
 * ==============================================
 *
 * (c) Copyright 2014-2016, by Juergen Walter, Fabian Gorsler, Fabian Brosig and other contributors.
 *
 * Project Info:   http://descartes.tools/dql
 *
 * All rights reserved. This software is made available under the terms of the
 * Eclipse Public License (EPL) v1.0 as published by the Eclipse Foundation
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * This software is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License (EPL)
 * for more details.
 *
 * You should have received a copy of the Eclipse Public License (EPL)
 * along with this software; if not visit http://www.eclipse.org or write to
 * Eclipse Foundation, Inc., 308 SW First Avenue, Suite 110, Portland, 97204 USA
 * Email: license (at) eclipse.org
 *
 * [Java is a trademark or registered trademark of Sun Microsystems, Inc.
 * in the United States and other countries.]
 */

package tools.descartes.pavo.export.csv;


import java.io.File;
import java.io.IOException;
import java.io.PrintWriter;


import org.jfree.chart.JFreeChart;

import tools.descartes.pavo.controller.IExportController;
import tools.descartes.pavo.result.AbstractQuantitativeResult;
import tools.descartes.pavo.result.ResultContainerPavo;
import tools.descartes.pavo.result.SeriesResultPavo;
import tools.descartes.pavo.util.Point;


/**
 * Export controller extension using the PAVO ExtensionPoint. 
 * The CSV file exported looks as follows:
 * 	first row: SeriesName, KindOfResult
 * 	other rows:  SeriesName, X, Y, ConfidenceIntervalHalfLength
 * 
 * @author Maximilian König <max.c.koenig@t-online.de>
 * @since 2016
 */

public class CSVExportControllerComma implements IExportController {

	


	public void export(File csvFile, JFreeChart chart, ResultContainerPavo result) {
		// SeriesName, KindOfResult
		// SeriesName, X, Y, ConfidenceIntervalHalfLength
		StringBuilder csv = new StringBuilder();
		for (AbstractQuantitativeResult r : result.getResults()) {
			if(r instanceof SeriesResultPavo)
			{
			for (int i = 0; i < ((SeriesResultPavo) r).getValues().size(); i++) {
				csv.append(writeLine(r.getQueryableElement() + r.getMetric() + r.getStatisticType(),
						((SeriesResultPavo) r).getValues().get(i)));

			}
			}

		}
		try {
			PrintWriter pw = new PrintWriter(csvFile);
			pw.write(csv.toString());
			pw.close();
		} catch (IOException e) {
			e.printStackTrace();
		}

	}

	
	private String writeLine(String string, Point p) {
		String conf = "";
		try{
			conf = p.getConf().getIntervalHalfLength().toString();
		}catch (NullPointerException e) {
		}
		String s = string + "," + p.getX() + "," + p.getY() + "," + conf+ "\n";
		return s;
	}

	@Override
	public String getStringForCombo() {
		return new String("CSV Export Komma");
	}

	@Override
	public String getIdentifier() {
		return new String("CSV Export Komma");
	}

	@Override
	public File updatePathToSuffix(File file) {
		String name = file.getPath();
		if (name.length() < 3 || !name.substring(name.length() - 3, name.length()).equals("csv")) {
			return new File(name + ".csv");
		} else {
			return file;
		}
	}

}