/**
 * ==============================================
 *  DQL : Descartes Query Language
 * ==============================================
 *
 * (c) Copyright 2014-2016, by Juergen Walter, Fabian Gorsler, Fabian Brosig and other contributors.
 *
 * Project Info:   http://descartes.tools/dql
 *
 * All rights reserved. This software is made available under the terms of the
 * Eclipse Public License (EPL) v1.0 as published by the Eclipse Foundation
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * This software is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License (EPL)
 * for more details.
 *
 * You should have received a copy of the Eclipse Public License (EPL)
 * along with this software; if not visit http://www.eclipse.org or write to
 * Eclipse Foundation, Inc., 308 SW First Avenue, Suite 110, Portland, 97204 USA
 * Email: license (at) eclipse.org
 *
 * [Java is a trademark or registered trademark of Sun Microsystems, Inc.
 * in the United States and other countries.]
 */
package tools.descartes.pavo.result;

import java.util.List;

import tools.descartes.dql.models.mapping.domain.Result;
import tools.descartes.pavo.util.Point;
import tools.descartes.pavo.util.dof;

/**
 * result class of PAVO for a Series
 * 
 * @author Maximilian König <max.c.koenig@t-online.de>
 * @since 2016
 */
public class SeriesResultPavo extends AbstractQuantitativeResult {

	/**
	 * Attribute for all points of this series
	 * 
	 * @see tools.descartes.pavo.util.Point
	 */
	private List<Point> values;

	/**
	 * Attribute showing if the current series is continous or not
	 */
	private Boolean isContinous;

	private String seriesName;

	private List<dof> dofs = null;

	/**
	 * @param queryableElementN
	 *            Queryable Element of the old EntityMapping
	 * @param metricN
	 *            current Metric
	 * @param statisticTypeN
	 *            current StatisticType
	 * @param scalexNeu
	 *            String for x axis
	 * @param scaleyNeu
	 *            String for y axis
	 * 
	 * @param pointsN
	 *            points of the series
	 * @param isContnousN
	 *            is the series continous?
	 * @param resultLocationNeu
	 *            location of the original result (needed for online update of
	 *            the result)
	 */
	public SeriesResultPavo(String queryableElementN, String metricN, String description, String statisticTypeN,
			String scalexNeu,
			String scaleyNeu, List<Point> pointsN, boolean isContnousN, Result resultLocation, List<dof> dofsN) {
		super(queryableElementN, metricN, description, statisticTypeN, scalexNeu, scaleyNeu, resultLocation);
		values = pointsN;
		isContinous = isContnousN;
		if (description.equals(null) || description == null)
			description="";
		if (!statisticTypeN.equalsIgnoreCase("DEFAULT") && !statisticTypeN.equalsIgnoreCase("SAMPLE") )
			seriesName = queryableElementN +"("+statisticTypeN+")";
		else
			seriesName = queryableElementN;
		dofs = dofsN;
	}


	/**
	 * @return returns true if the series contains values < 0
	 */
	public boolean hasNegatives() {
		for (Point p : values) {
			if (p.getX() < 0 && p.getY() < 0)
				return true;
		}
		return false;

	}

	/**
	 * @param cmp
	 *            series to compare
	 * @return returns true if the current series contains the same points as
	 *         the cmp series
	 */
	public boolean isSame(SeriesResultPavo cmp) {
		if (this.values.size() != cmp.getValues().size())
			return false;

		for (int i = 0; i < this.values.size(); i++) {
			if (!this.values.get(i).equals(cmp.getValues().get(i)))
				return false;
		}
		return true;

	}

	/**
	 * @return the values
	 */
	public List<Point> getValues() {
		return values;
	}

	/**
	 * @param values
	 *            the values to set
	 */
	public void setValues(List<Point> values) {
		this.values = values;
	}

	/**
	 * @return the isContinous
	 */
	public Boolean isContinous() {
		return isContinous;
	}

	/**
	 * @param isContinous
	 *            the isContinous to set
	 */
	public void setIsContinous(Boolean isContinous) {
		this.isContinous = isContinous;
	}

	public String getSeriesName() {
		return seriesName;
	}

	public void setSeriesName(String seriesName) {
		this.seriesName = seriesName;
	}

	public List<dof> getDofs() {
		return dofs;
	}

	public void setDofs(List<dof> dofs) {
		this.dofs = dofs;
	}
}
