/**
 * ==============================================
 *  DQL : Descartes Query Language
 * ==============================================
 *
 * (c) Copyright 2014-2016, by Juergen Walter, Fabian Gorsler, Fabian Brosig and other contributors.
 *
 * Project Info:   http://descartes.tools/dql
 *
 * All rights reserved. This software is made available under the terms of the
 * Eclipse Public License (EPL) v1.0 as published by the Eclipse Foundation
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * This software is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License (EPL)
 * for more details.
 *
 * You should have received a copy of the Eclipse Public License (EPL)
 * along with this software; if not visit http://www.eclipse.org or write to
 * Eclipse Foundation, Inc., 308 SW First Avenue, Suite 110, Portland, 97204 USA
 * Email: license (at) eclipse.org
 *
 * [Java is a trademark or registered trademark of Sun Microsystems, Inc.
 * in the United States and other countries.]
 */
package tools.descartes.pavo.types;

import java.awt.Color;
import java.util.LinkedList;
import java.util.List;

import javax.swing.JPanel;
import javax.swing.JRadioButton;

import org.jfree.chart.JFreeChart;
import org.jfree.chart.axis.CategoryAxis;
import org.jfree.chart.axis.NumberAxis;
import org.jfree.chart.plot.CategoryPlot;
import org.jfree.chart.renderer.category.BoxAndWhiskerRenderer;
import org.jfree.data.Range;
import org.jfree.data.statistics.DefaultBoxAndWhiskerCategoryDataset;

import tools.descartes.pavo.controller.VisualizationController;
import tools.descartes.pavo.result.AbstractQuantitativeResult;
import tools.descartes.pavo.result.ResultContainerPavo;
import tools.descartes.pavo.result.SeriesResultPavo;
import tools.descartes.pavo.util.Point;
import tools.descartes.pavo.view.View;

/**
 * * Class to visualise the current result as BoxPlot if possible. Can visualise
 * results constiting out of value results or series results (Values are shown
 * as horizontal markers). All x values are ignored and a box plot for the y
 * values is shown. Does not do any configurations for the control buttons
 * 
 * @author Maximilian König <max.c.koenig@t-online.de>
 * @since 2016
 */
public class BoxPlot extends AbstractVisualizationType implements IVisualizationType {

	public BoxPlot(ResultContainerPavo resultN) {
		super(resultN);
	}

	@Override
	public JFreeChart visualize() {
		DefaultBoxAndWhiskerCategoryDataset dataset = new DefaultBoxAndWhiskerCategoryDataset();
		final CategoryAxis xAxis = new CategoryAxis();
		final NumberAxis yAxis = new NumberAxis(this.getYDescription());
		BoxAndWhiskerRenderer renderer = new BoxAndWhiskerRenderer();
		renderer.setMeanVisible(false);
		CategoryPlot plot = new CategoryPlot(dataset, xAxis, yAxis, renderer);
		JFreeChart chart = new JFreeChart(plot);

		for (AbstractQuantitativeResult r : getResult().getResults()) {
			if (r instanceof SeriesResultPavo) {
				List<Double> list = new LinkedList<Double>();
				for (Point p : ((SeriesResultPavo) r).getValues()) {
					list.add(p.getY());
				}
				dataset.add(list, ((SeriesResultPavo) r).getSeriesName(), "");
			}
		}

		plot.setRangeGridlinePaint(Color.lightGray);
		plot.setDomainGridlinePaint(Color.lightGray);
		NumberAxis axis = ((NumberAxis)plot.getRangeAxis());
		axis.setRange(new Range(0, axis.getRange().getUpperBound()*1.4));
		axis.setUpperBound(axis.getRange().getUpperBound()*1.4);
		return chart;
	}

	@Override
	public boolean canVisualize() {
		for (AbstractQuantitativeResult r : this.getResult().getResults()) {
			if (!(r instanceof SeriesResultPavo)) {
				return false;
			}
		}
		if (this.getResult().getResults().size() < 1)
			return false;
		return true;
	}

	public JPanel getEditViewComponent(VisualizationController c) {
		JPanel panel = new JPanel();
		JRadioButton a = View.createConfidence(c.getActionListenerConfidence());
		a.setEnabled(false);
		panel.add(a);
		return panel;
	}

}