/**
 * ==============================================
 *  DQL : Descartes Query Language
 * ==============================================
 *
 * (c) Copyright 2014-2016, by Juergen Walter, Fabian Gorsler, Fabian Brosig and other contributors.
 *
 * Project Info:   http://descartes.tools/dql
 *
 * All rights reserved. This software is made available under the terms of the
 * Eclipse Public License (EPL) v1.0 as published by the Eclipse Foundation
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * This software is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License (EPL)
 * for more details.
 *
 * You should have received a copy of the Eclipse Public License (EPL)
 * along with this software; if not visit http://www.eclipse.org or write to
 * Eclipse Foundation, Inc., 308 SW First Avenue, Suite 110, Portland, 97204 USA
 * Email: license (at) eclipse.org
 *
 * [Java is a trademark or registered trademark of Sun Microsystems, Inc.
 * in the United States and other countries.]
 */
package tools.descartes.pavo.types;

import java.awt.Color;

import org.jfree.chart.JFreeChart;
import org.jfree.chart.axis.NumberAxis;
import org.jfree.chart.axis.ValueAxis;
import org.jfree.chart.plot.ValueMarker;
import org.jfree.chart.plot.XYPlot;
import org.jfree.chart.renderer.xy.XYDifferenceRenderer;
import org.jfree.data.xy.XYSeries;
import org.jfree.data.xy.XYSeriesCollection;
import org.jfree.ui.RectangleAnchor;
import org.jfree.ui.TextAnchor;

import tools.descartes.pavo.result.AbstractQuantitativeResult;
import tools.descartes.pavo.result.ResultContainerPavo;
import tools.descartes.pavo.result.SeriesResultPavo;
import tools.descartes.pavo.result.ValueResultPavo;
import tools.descartes.pavo.util.Point;

/**
 * Class to visualise the current result as DifferenceChart if possible. Can visualise
 * results constiting out of 2 series results, one series and one value result or two value results (Values are shown
 * as horizontal markers). Does not do any configurations for the control
 * buttons
 * 
 * @author Maximilian König <max.c.koenig@t-online.de>
 * @since 2016
 */
public class DifferenceChart extends AbstractVisualizationType implements IVisualizationType {

	private double maxValueSeries = 10;

	public DifferenceChart(ResultContainerPavo result) {
		this.setResult(result);
		for (AbstractQuantitativeResult result1 : result.getResults()) {

			if (result1 instanceof SeriesResultPavo) {
				maxValueSeries = ((SeriesResultPavo) result1).getValues()
						.get(((SeriesResultPavo) result1).getValues().size() - 1).getX();

			}
		}
	}

	@Override
	public boolean canVisualize() {

		int counterSeries = 0;
		int valueCounter = 0;
		if (this.getResult().getResults().size() != 2) {
			return false;
		}
		for (AbstractQuantitativeResult result : this.getResult().getResults()) {
			if (result instanceof SeriesResultPavo)
				counterSeries++;
			else if (result instanceof ValueResultPavo)
				valueCounter++;
			if (!(result instanceof ValueResultPavo) && !(result instanceof SeriesResultPavo)) // &&
																								// ((SeriesResultPavo)
																								// result).isContinous()))//Wunsch
																								// Juergen
			{
				return false;

			}
		}
		if (counterSeries != 2 && !(valueCounter == 1 && counterSeries == 1) && valueCounter != 2) {
			return false;
		}
		return true;
	}

	@Override
	public JFreeChart visualize() {

		XYSeriesCollection dataset = new XYSeriesCollection();
		NumberAxis domainAxis = new NumberAxis(this.getXDescription());
		domainAxis.setAutoRangeIncludesZero(false);
		NumberAxis valueAxis = new NumberAxis(this.getYDescription());
		final XYDifferenceRenderer renderer = new XYDifferenceRenderer(new Color(193, 204, 204, 80),
				new Color(193, 204, 204, 80), false);
		renderer.setSeriesPaint(0, Color.yellow);
		renderer.setSeriesPaint(1, Color.red);
		XYPlot plot = new XYPlot(dataset, domainAxis, valueAxis, renderer);
		JFreeChart chart = new JFreeChart("", JFreeChart.DEFAULT_TITLE_FONT, plot, true);
		for (AbstractQuantitativeResult r : this.getResult().getResults()) {
			if (r instanceof ValueResultPavo) {
				ValueMarker marker = new ValueMarker(((ValueResultPavo) r).getValue());
				marker.setPaint(Color.green);
				marker.setLabel(((ValueResultPavo) r).getStatisticType());
				marker.setLabelAnchor(RectangleAnchor.BOTTOM_RIGHT);
				marker.setLabelTextAnchor(TextAnchor.TOP_RIGHT);
				((XYPlot) chart.getPlot()).addRangeMarker(marker);
				ValueAxis yAxis = plot.getRangeAxis();
				if (marker.getValue() > yAxis.getUpperBound())
					yAxis.setRange(yAxis.getLowerBound(), marker.getValue() + 1);
				XYSeries series1 = new XYSeries(r.getQueryableElement());

				series1.add(0, ((ValueResultPavo) r).getValue());
				series1.add(this.maxValueSeries, ((ValueResultPavo) r).getValue());
				dataset.addSeries(series1);
			} else {
				XYSeries series1 = new XYSeries(((SeriesResultPavo) r).getSeriesName());
				for (Point p : ((SeriesResultPavo) r).getValues()) {
					series1.add(p.getX(), p.getY());
				}
				dataset.addSeries(series1);
			}

		}
		plot.setBackgroundPaint(Color.white);
		plot.setRangeGridlinePaint(Color.lightGray);
		plot.setDomainGridlinePaint(Color.lightGray);
		return chart;
	}

}
