package tools.descartes.pavo.types;
/**
 * ==============================================
 *  DQL : Descartes Query Language
 * ==============================================
 *
 * (c) Copyright 2014-2016, by Juergen Walter, Fabian Gorsler, Fabian Brosig and other contributors.
 *
 * Project Info:   http://descartes.tools/dql
 *
 * All rights reserved. This software is made available under the terms of the
 * Eclipse Public License (EPL) v1.0 as published by the Eclipse Foundation
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * This software is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License (EPL)
 * for more details.
 *
 * You should have received a copy of the Eclipse Public License (EPL)
 * along with this software; if not visit http://www.eclipse.org or write to
 * Eclipse Foundation, Inc., 308 SW First Avenue, Suite 110, Portland, 97204 USA
 * Email: license (at) eclipse.org
 *
 * [Java is a trademark or registered trademark of Sun Microsystems, Inc.
 * in the United States and other countries.]
 */

import java.awt.Color;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.LinkedList;
import java.util.List;

import javax.swing.BoxLayout;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;

import org.jfree.chart.JFreeChart;
import org.jfree.chart.axis.NumberAxis;
import org.jfree.chart.plot.XYPlot;
import org.jfree.chart.renderer.GrayPaintScale;
import org.jfree.chart.renderer.PaintScale;
import org.jfree.chart.renderer.xy.XYBlockRenderer;
import org.jfree.data.DomainOrder;
import org.jfree.data.general.DatasetChangeListener;
import org.jfree.data.general.DatasetGroup;
import org.jfree.data.xy.XYZDataset;

import tools.descartes.pavo.controller.VisualizationController;
import tools.descartes.pavo.result.AbstractQuantitativeResult;
import tools.descartes.pavo.result.ResultContainerPavo;
import tools.descartes.pavo.result.ValueResultPavo;
import tools.descartes.pavo.util.dof;

/**
 * Class to visualise the current result as HeatMap if possible. Can visualise
 * results constiting out at least 3 series of values. Removing the confidence
 * button and adding three dropdown lists for configure the x, y and z series
 * for the HeatMap chart
 * 
 * @author Maximilian König <max.c.koenig@t-online.de>
 * @since 2016
 */
public class HeatMap extends AbstractVisualizationType implements IVisualizationType {

	private JComboBox<String> comboX;
	private JComboBox<String> comboY;
	private String actualX;
	private String actualY;
	private JPanel panelX;
	private JPanel panelY;
	boolean blocker = false;
	private VisualizationController vizControl;
	LinkedList<List<Double>> seriesList = new LinkedList<List<Double>>();
	LinkedList<String> seriesNamelist = new LinkedList<String>();

	LinkedList<String> doflist = new LinkedList<String>();

	public HeatMap(ResultContainerPavo resultN, VisualizationController v) {
		super(resultN);
		this.vizControl = v;
		if (this.canVisualize()) {
			panelX = new JPanel();
			panelX.setLayout(new FlowLayout());
			panelY = new JPanel();
			panelY.setLayout(new FlowLayout());

			panelX.add(new JLabel("X:"));
			panelY.add(new JLabel("Y:"));
			comboX = new JComboBox<String>();
			comboY = new JComboBox<String>();
			comboX.setToolTipText("X Axis DoF");
			comboY.setToolTipText("Y Axis DoF");

			panelX.add(comboX);
			panelY.add(comboY);
			setActionListeners();
			for (AbstractQuantitativeResult r : this.getResult().getResults()) {
				if (r instanceof ValueResultPavo) {
					{
						for (dof d : ((ValueResultPavo) r).getDofs()) {
							if (!doflist.contains(d.getDofname()))
								doflist.add(d.getDofname());
						}
					}
				}
			}

			blocker = true;
			this.fillComboBoxes(doflist.toArray());
			blocker = false;
			updateCurrentXYZ();

		}

	}

	@Override
	public JFreeChart visualize() {
		NumberAxis xAxis = new NumberAxis(this.actualX);

		NumberAxis yAxis = new NumberAxis(this.actualY);

		XYBlockRenderer renderer = new XYBlockRenderer();
		PaintScale scale = new GrayPaintScale(-2.0, 1.0);

		renderer.setPaintScale(scale);

		XYZDataset dataset = new XYZDataset() {
			public int getSeriesCount() {
				return 1;
			}

			public int getItemCount(int series) {
				return getResult().getResults().size();
			}

			public Number getX(int series, int item) {

				return new Double(getXValue(series, item));
			}

			public double getXValue(int series, int item) {
				// Hier x dof nehmen
				for (dof d : ((ValueResultPavo) getResult().getResults().get(item)).getDofs()) {
					if (d.getDofname().equals(actualX)) {
						return d.getDofvalue();
					}
				}
				return 0;
			}

			public Number getY(int series, int item) {
				return new Double(getYValue(series, item));
			}

			public double getYValue(int series, int item) {
				for (dof d : ((ValueResultPavo) getResult().getResults().get(item)).getDofs()) {
					if (d.getDofname().equals(actualY)) {
						return d.getDofvalue();
					}
				}
				return 0;
			}

			public Number getZ(int series, int item) {
				return new Double(getZValue(series, item));
			}

			public double getZValue(int series, int item) {
				return ((ValueResultPavo) getResult().getResults().get(item)).getValue();
			}

			public void addChangeListener(DatasetChangeListener listener) {
				// ignore - this dataset never changes
			}

			public void removeChangeListener(DatasetChangeListener listener) {
				// ignore
			}

			public DatasetGroup getGroup() {
				return null;
			}

			public void setGroup(DatasetGroup group) {
				// ignore
			}

			@SuppressWarnings("rawtypes")
			public Comparable getSeriesKey(int series) {
				return getResult().getMetric();
			}

			@SuppressWarnings("rawtypes")
			public int indexOf(Comparable seriesKey) {
				return 0;
			}

			public DomainOrder getDomainOrder() {
				return DomainOrder.ASCENDING;
			}
		};

		XYPlot plot = new XYPlot(dataset, xAxis, yAxis, renderer);

		plot.setBackgroundPaint(Color.white);
		plot.setRangeGridlinePaint(Color.lightGray);
		plot.setDomainGridlinePaint(Color.lightGray);
		JFreeChart chart = new JFreeChart("", plot);
		return chart;
	}

	@Override
	public boolean canVisualize() {

		double lastdof1 = Double.MIN_VALUE;
		double lastdof2 = Double.MIN_VALUE;
		List<String> dofs = new LinkedList<String>();
		for (AbstractQuantitativeResult r : this.getResult().getResults()) {
			if (!(r instanceof ValueResultPavo)) {
				return false;

			}

			for (dof d : ((ValueResultPavo) r).getDofs()) {
				boolean contains = false;
				for (String s : dofs) {
					if (s.equals(d.getDofname()))
						contains = true;

				}
				if (!contains)
					dofs.add(d.getDofname());
			}
			if (((ValueResultPavo) r).getDofs().size() != 2) {
				return false;
			}
		}
		for (AbstractQuantitativeResult r : this.getResult().getResults()) {

			if (lastdof1 == Double.MIN_VALUE)
				lastdof1 = ((ValueResultPavo) r).getDofs().get(0).getDofvalue();
			else {
				if (Math.abs(lastdof1 - ((ValueResultPavo) r).getDofs().get(0).getDofvalue()) > 1) {
					return false;
				}

			}
			if (lastdof2 == Double.MIN_VALUE)
				lastdof2 = ((ValueResultPavo) r).getDofs().get(1).getDofvalue();
			else {
				if (Math.abs(lastdof2 - ((ValueResultPavo) r).getDofs().get(1).getDofvalue()) > 1) {
					return false;
				}
			}

		}
		return true;
	}

	@Override
	public JPanel getEditViewComponent(VisualizationController c) {

		JPanel panel = new JPanel();
		panel.setLayout(new BoxLayout(panel, BoxLayout.Y_AXIS));
		panel.add(panelX);
		panel.add(panelY);
		updateCurrentXYZ();
		blocker = true;
		this.fillComboBoxes(doflist.toArray());
		blocker = false;
		return panel;
	}

	/**
	 * @return the blockerHeat
	 */
	public boolean isBlockerHeat() {
		return blocker;
	}

	/**
	 * @param blockerHeat
	 *            the blockerHeat to set
	 */
	public void setBlockerHeat(boolean blockerHeat) {
		this.blocker = blockerHeat;
	}

	public void fillComboBoxes(Object[] array) {
		comboX.removeAllItems();
		comboY.removeAllItems();
		for (int i = 0; i < array.length; i++) {
			comboX.addItem(array[i].toString());
			comboY.addItem(array[i].toString());
		}
		comboX.setSelectedIndex(0);
		comboY.setSelectedIndex(1);
	}

	public void updateCurrentXYZ() {
		actualX = comboX.getSelectedItem().toString();
		actualY = comboY.getSelectedItem().toString();
	}

	public void setActionListeners() {

		comboX.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent e) {
				if (blocker == true) {

				} else {
					blocker = true;
					String now = comboX.getSelectedItem().toString();
					if (comboY.getSelectedItem().toString().equals(now)) {
						comboY.removeAllItems();
						comboY.addItem(actualX);

						for (String s : doflist) {
							if (!s.equals(actualX))
								comboY.addItem(s);
						}
						updateCurrentXYZ();

					}
					blocker = false;
					vizControl.updateChart(visualize());
				}

			}

		});

		comboY.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent e) {
				if (blocker == true) {

				} else {
					blocker = true;
					String now = comboY.getSelectedItem().toString();
					if (comboX.getSelectedItem().toString().equals(now)) {
						comboX.removeAllItems();
						comboX.addItem(actualY);

						for (String s : doflist) {
							if (!s.equals(actualY))
								comboX.addItem(s);
						}
						updateCurrentXYZ();
						blocker = false;
						vizControl.updateChart(visualize());
					}

				}
			}
		});
	}

}