/**
 * ==============================================
 *  DQL : Descartes Query Language
 * ==============================================
 *
 * (c) Copyright 2014-2016, by Juergen Walter, Fabian Gorsler, Fabian Brosig and other contributors.
 *
 * Project Info:   http://descartes.tools/dql
 *
 * All rights reserved. This software is made available under the terms of the
 * Eclipse Public License (EPL) v1.0 as published by the Eclipse Foundation
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * This software is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License (EPL)
 * for more details.
 *
 * You should have received a copy of the Eclipse Public License (EPL)
 * along with this software; if not visit http://www.eclipse.org or write to
 * Eclipse Foundation, Inc., 308 SW First Avenue, Suite 110, Portland, 97204 USA
 * Email: license (at) eclipse.org
 *
 * [Java is a trademark or registered trademark of Sun Microsystems, Inc.
 * in the United States and other countries.]
 */

package tools.descartes.pavo.types;

import java.awt.Color;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.LinkedList;

import javax.swing.BoxLayout;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JSpinner;
import javax.swing.SpinnerNumberModel;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import org.jfree.chart.ChartFactory;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.chart.plot.XYPlot;
import org.jfree.data.statistics.HistogramDataset;
import org.jfree.data.statistics.HistogramType;

import tools.descartes.pavo.controller.VisualizationController;
import tools.descartes.pavo.result.AbstractQuantitativeResult;
import tools.descartes.pavo.result.ResultContainerPavo;
import tools.descartes.pavo.result.SeriesResultPavo;

/**
 * Class to visualize the current series results as histogram. Removes the
 * confidence button and adding a spinner for the choice of number of bins
 * 
 * @author Jrgen Walter <juergen.walter@uni-wuerzburg.de>
 * @author Maximilian Knig <max.c.koenig@t-online.de>
 * @since 2016
 */
public class Historgram extends AbstractVisualizationType implements IVisualizationType {

	JRadioButton equidistant;
	JRadioButton equalnumber;
	JSpinner mySpinner;

	int numberOfBins = 5;

	public Historgram(ResultContainerPavo resultN) {
		super(resultN);

	}

	@Override
	public JFreeChart visualize() {

		HistogramDataset dataset = new HistogramDataset();
		dataset.setType(HistogramType.RELATIVE_FREQUENCY);

		for (AbstractQuantitativeResult r : this.getResult().getResults()) {
			if (r instanceof SeriesResultPavo) {
				LinkedList<Double> list = new LinkedList<>();
				for (int i = 0; i < ((SeriesResultPavo) r).getValues().size(); i++) {
					list.add(((SeriesResultPavo) r).getValues().get(i).getY());

				}

				double[] array = new double[list.size()];
				for (int i = 0; i < list.size(); i++) {
					array[i] = list.get(i); // Watch out for
											// NullPointerExceptions!
				}
				dataset.addSeries(((SeriesResultPavo) r).getSeriesName(), array, numberOfBins);
			}
		}

		String xaxis = this.getYDescription();
		String yaxis = "Frequency";
		JFreeChart chart = ChartFactory.createHistogram("", xaxis, yaxis, dataset, PlotOrientation.VERTICAL, false,
				false, false);
		XYPlot plot = (XYPlot) chart.getPlot();
		plot.setBackgroundPaint(Color.white);
		plot.setRangeGridlinePaint(Color.lightGray);
		plot.setDomainGridlinePaint(Color.lightGray);
		return chart;

	}

	@Override
	public boolean canVisualize() {
		for (AbstractQuantitativeResult r : this.getResult().getResults()) {
			if (!(r instanceof SeriesResultPavo)) {
				return false;
			}
		}
		return true;

	}

	@Override
	public JPanel getEditViewComponent(VisualizationController c) {
		JPanel panel = new JPanel();
		panel.setLayout(new BoxLayout(panel, BoxLayout.Y_AXIS));
		equidistant = new JRadioButton("Equidistant");
		equidistant.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent e) {
				equalnumber.setSelected(!equidistant.isSelected());
				c.draw(Historgram.class, false);

			}
		});
		equalnumber = new JRadioButton("Equal number of elements");
		equalnumber.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent e) {
				equidistant.setSelected(!equalnumber.isSelected());
				c.draw(Historgram.class, false);
			}
		});

		mySpinner = new JSpinner();
		mySpinner.setModel(new SpinnerNumberModel(numberOfBins, 1,
				((SeriesResultPavo) this.getResult().getResults().get(0)).getValues().size(), 1));
		mySpinner.addChangeListener(new ChangeListener() {

			@Override
			public void stateChanged(ChangeEvent e) {
				numberOfBins = (int) mySpinner.getValue();
				c.updateChart(visualize());

			}
		});

		equalnumber.setVisible(false);
		equidistant.setVisible(false);

		panel.add(equalnumber);
		panel.add(equidistant);
		panel.add(mySpinner);

		return panel;
	}

}