/**
 * ==============================================
 *  DQL : Descartes Query Language
 * ==============================================
 *
 * (c) Copyright 2014-2016, by Juergen Walter, Fabian Gorsler, Fabian Brosig and other contributors.
 *
 * Project Info:   http://descartes.tools/dql
 *
 * All rights reserved. This software is made available under the terms of the
 * Eclipse Public License (EPL) v1.0 as published by the Eclipse Foundation
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * This software is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License (EPL)
 * for more details.
 *
 * You should have received a copy of the Eclipse Public License (EPL)
 * along with this software; if not visit http://www.eclipse.org or write to
 * Eclipse Foundation, Inc., 308 SW First Avenue, Suite 110, Portland, 97204 USA
 * Email: license (at) eclipse.org
 *
 * [Java is a trademark or registered trademark of Sun Microsystems, Inc.
 * in the United States and other countries.]
 */
package tools.descartes.pavo.view;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.SwingConstants;
import javax.swing.border.EmptyBorder;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExtensionRegistry;
import org.eclipse.core.runtime.Platform;
import org.jfree.chart.JFreeChart;

import tools.descartes.pavo.controller.IExportController;
import tools.descartes.pavo.result.ResultContainerPavo;

/**
 * Code managing the seperate view, which opens, by clicking the export button
 * on the main view. Containing the combobox for the type of export, the file
 * selection and the finish/cancel button. This class works with the different
 * extensions for the extensionpoint provided by PAVO
 * 
 * @author Maximilian König <max.c.koenig@t-online.de>
 * @since 2016
 */

public class ExportDialogFrame extends JFrame {

	private JFrame frame = this;

	private static JFileChooser c = new JFileChooser();
	private static final long serialVersionUID = 1L;
	private JPanel contentPane;
	private JLabel header;
	private JLabel widthLabel;
	private JLabel heightLabel;
	private JLabel infoCSV;
	private JButton finishButton;
	private JButton cancelButton = new JButton("Cancel");;
	private JTextField height;
	private JTextField width;
	private double factorWidthToHeight;
	private boolean saveSuccessful = false;
	private JComboBox<String> kindOfExport = new JComboBox<String>();
	private JFreeChart chartToExport;
	public static int heightStatic;
	public static int widthStatic;
	public ResultContainerPavo result;

	/**
	 * constructor to create a new window for the export selection
	 * 
	 * @param factorWidthToHeightN
	 *            the factor for the width and height for the shown chart. to
	 *            prevent bias
	 * @param chartN
	 *            the chart that is currently shown
	 * @param resultN
	 *            the result that is shown in the chart (needed for SVG export)
	 */
	/**
	 * @param factorWidthToHeightN
	 * @param chartN
	 * @param resultN
	 */
	public ExportDialogFrame(final double factorWidthToHeightN, JFreeChart chartN, ResultContainerPavo resultN) {
		chartToExport = chartN;
		factorWidthToHeight = factorWidthToHeightN;
		result = resultN;
		initializeGUIComponents();
		updateHeightFromWidth();
		initActionListenerFinish();
		initActionListenerCancel();

		heightStatic = getHeightFromText();
		widthStatic = getWidthFromText();

		IExtensionRegistry registry = Platform.getExtensionRegistry();
		IConfigurationElement[] config = registry.getConfigurationElementsFor("tools.descartes.pavo.export");
		for (IConfigurationElement e : config) {
			try {
				final Object o = e.createExecutableExtension("class");
				if (o instanceof IExportController) {
					this.kindOfExport.addItem(((IExportController) o).getStringForCombo());
				}
			} catch (CoreException ex) {
				System.out.println(ex.getMessage());
			}

		}

		initExportDialog();

		this.setVisible(true);
		this.addActionListener();
	}

	/**
	 * Method to initialize the shown controls depending on which export type is
	 * currently shown in the combobox
	 */
	private void initExportDialog() {
		if (kindOfExport.getSelectedItem().equals("PNG Export")) {
			infoCSV.setVisible(false);
			widthLabel.setVisible(true);
			heightLabel.setVisible(true);
			width.setVisible(true);
			height.setVisible(true);
		} else if (kindOfExport.getSelectedItem().equals("SVG Export")) {
			infoCSV.setVisible(false);
			widthLabel.setVisible(true);
			heightLabel.setVisible(true);
			width.setVisible(true);
			height.setVisible(true);
		} else if (kindOfExport.getSelectedItem().equals("PDF Export")) {
			width.setText("800");
			updateHeightFromWidth();
			infoCSV.setVisible(false);
			widthLabel.setVisible(true);
			heightLabel.setVisible(true);
			width.setVisible(true);
			height.setVisible(true);
		} else {
			infoCSV.setVisible(true);
			widthLabel.setVisible(false);
			heightLabel.setVisible(false);
			width.setVisible(false);
			height.setVisible(false);
		}
		height.setEnabled(false);
	}

	/**
	 * Method, initializing all components of this view
	 */
	private void initializeGUIComponents() {
		infoCSV = new JLabel("You have chosen the CSV Export");

		setBounds(100, 100, 450, 300);
		contentPane = new JPanel();
		contentPane.setPreferredSize(new Dimension(400, 300));
		contentPane.setMaximumSize(new Dimension(400, 300));
		contentPane.setMinimumSize(new Dimension(400, 300));
		contentPane.setBorder(new EmptyBorder(5, 5, 5, 5));
		contentPane.setLayout(new BorderLayout(0, 0));
		setContentPane(contentPane);

		header = new JLabel("Select your type of Export for the chart!");
		header.setVerticalAlignment(SwingConstants.TOP);
		header.setMinimumSize(new Dimension(327, 70));
		header.setPreferredSize(new Dimension(327, 70));
		header.setHorizontalAlignment(SwingConstants.CENTER);
		JPanel helpHeader = new JPanel();
		helpHeader.setLayout(new BoxLayout(helpHeader, BoxLayout.Y_AXIS));
		helpHeader.add(header);
		helpHeader.add(kindOfExport);
		contentPane.add(helpHeader, BorderLayout.NORTH);

		contentPane.add(infoCSV);
		Box verticalBox = Box.createVerticalBox();
		contentPane.add(verticalBox, BorderLayout.CENTER);
		Box horizontalBox_1 = Box.createHorizontalBox();
		verticalBox.add(horizontalBox_1);
		widthLabel = new JLabel("Selected Width:             ");
		horizontalBox_1.add(widthLabel);
		width = new JTextField("");
		width.setMaximumSize(new Dimension(200, 30));
		width.setMinimumSize(new Dimension(200, 30));
		horizontalBox_1.add(width);
		width.setText("1000");
		width.setPreferredSize(new Dimension(200, 30));
		Box horizontalBox = Box.createHorizontalBox();
		verticalBox.add(horizontalBox);
		heightLabel = new JLabel("Selected Height:           ");
		horizontalBox.add(heightLabel);
		height = new JTextField("");
		height.setPreferredSize(new Dimension(200, 30));
		height.setMaximumSize(new Dimension(200, 30));
		height.setMinimumSize(new Dimension(200, 30));
		horizontalBox.add(height);
		height.setColumns(10);
		JPanel panel = new JPanel();
		contentPane.add(panel, BorderLayout.SOUTH);
		finishButton = new JButton("Finish Save");
		panel.add(finishButton);
		panel.add(cancelButton);

		JPanel panel_1 = new JPanel();
		panel_1.setPreferredSize(new Dimension(50, 10));
		panel_1.setMinimumSize(new Dimension(50, 10));
		panel.add(panel_1);
	}

	/**
	 * Method to add the actionlistener to the cancelbutton
	 */
	private void initActionListenerCancel() {
		cancelButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				close();
			}
		});
	}

	/**
	 * Method to add the actionlistener to the finish button
	 */
	private void initActionListenerFinish() {
		finishButton.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent e) {
				setSaveSuccessful(true);

				int a = c.showSaveDialog(contentPane);

				if (a == JFileChooser.CANCEL_OPTION) {
					System.out.println("Canceled Save Action!");
				} else {
					final File file = c.getSelectedFile();
					File fileToUse;

					String selectedExport = kindOfExport.getSelectedItem().toString();

					IExtensionRegistry registry = Platform.getExtensionRegistry();
					IConfigurationElement[] config = registry
							.getConfigurationElementsFor("tools.descartes.pavo.export");
					try {
						for (IConfigurationElement f : config) {
							final Object o = f.createExecutableExtension("class");
							if (((IExportController) o).getIdentifier().equals(selectedExport)) {
								if (getSuccess()) {
									fileToUse = ((IExportController) o).updatePathToSuffix(file); // Update
																									// Name
																									// der
																									// Datei
																									// und
																									// Export
									((IExportController) o).export(fileToUse, chartToExport, result);
								}
							}

						}
						frame.dispose();

					} catch (CoreException ex) {
						System.out.println(ex.getMessage());
					}

				}
			}
		});
	}

	/**
	 * Method updating the choosen height depending on the width written by the
	 * user and the factor of width and height
	 */
	public void updateHeightFromWidth() {
		finishButton.setEnabled(true);
		try {
			double x = Double.parseDouble(width.getText()) * this.factorWidthToHeight;
			int y = (int) x;
			height.setText(String.valueOf(y));
		} catch (java.lang.NumberFormatException e) {
			height.setText("ERROR");
			finishButton.setEnabled(false);
		}
	}

	/**
	 * close this view
	 */
	public void close() {
		this.dispose();
	}

	/**
	 * @return the width shown in the textfield
	 */
	public int getWidthFromText() {
		return new Integer(width.getText());
	}

	/**
	 * @return the height shown in the textfield
	 */
	public int getHeightFromText() {
		return new Integer(height.getText());
	}

	/**
	 * @return true if the save was succesful (a file was selected)
	 */
	public boolean getSuccess() {
		return saveSuccessful;
	}

	/**
	 * Method to add actionlisteners to the width textfield and the kind of
	 * export combobox
	 */
	public void addActionListener() {
		width.getDocument().addDocumentListener(new DocumentListener() {

			@Override
			public void removeUpdate(DocumentEvent e) {
				updateHeightFromWidth();
				if (!height.getText().equals("ERROR")) {
					heightStatic = getHeightFromText();
					widthStatic = getWidthFromText();
				}

			}

			@Override
			public void insertUpdate(DocumentEvent e) {
				updateHeightFromWidth();
				heightStatic = getHeightFromText();
				widthStatic = getWidthFromText();

			}

			@Override
			public void changedUpdate(DocumentEvent e) {
				updateHeightFromWidth();
				heightStatic = getHeightFromText();
				widthStatic = getWidthFromText();

			}
		});

		kindOfExport.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent e) {
				initExportDialog();

			}
		});
	}

	/**
	 * @return the kindOfExport
	 */
	public JComboBox<String> getKindOfExport() {
		return this.kindOfExport;
	}

	/**
	 * @param kindOfExport
	 *            the kindOfExport to set
	 */
	public void setKindOfExport(JComboBox<String> kindOfExport) {
		this.kindOfExport = kindOfExport;
	}

	/**
	 * @return the finishButton
	 */
	public JButton getFinishButton() {
		return finishButton;
	}

	/**
	 * @param finishButton
	 *            the finishButton to set
	 */
	public void setFinishButton(JButton finishButton) {
		this.finishButton = finishButton;
	}

	/**
	 * @return the saveSuccessful
	 */
	public boolean isSaveSuccessful() {
		return saveSuccessful;
	}

	/**
	 * @param saveSuccessful
	 *            the saveSuccessful to set
	 */
	public void setSaveSuccessful(boolean saveSuccessful) {
		this.saveSuccessful = saveSuccessful;
	}
}
